@@ -126,106 +126,163 @@ GeomSf <- ggproto("GeomSf", Geom,
     fill = NULL,
     size = NULL,
     linewidth = NULL,
-    linetype = 1,
+    linetype = from_theme(linetype),
     alpha = NA,
     stroke = 0.5
   ),
 
+  use_defaults = function(self, data, params = list(), modifiers = aes(),
+                          default_aes = NULL, theme = NULL, ...) {
+    data <- ggproto_parent(Geom, self)$use_defaults(
+      data, params, modifiers, default_aes, theme = theme, ...
+    )
+    # Early exit for e.g. legend data that don't have geometry columns
+    if (!"geometry" %in% names(data)) {
+      return(data)
+    }
+
+    # geometry column is a character if we're populating legend keys
+    type <- if (is.character(data$geometry)) {
+      data$geometry
+    } else {
+      sf_types[sf::st_geometry_type(data$geometry)]
+    }
+
+    # Devise splitting index for geometry types
+    type <- factor(type, c("point", "line", "other", "collection"))
+    index <- split(seq_len(nrow(data)), type)
+
+    # Initialise parts of the data
+    points <- lines <- others <- collections <- NULL
+
+    # Go through every part, applying different defaults
+    if (length(index$point) > 0) {
+      points <- GeomPoint$use_defaults(
+        vec_slice(data, index$point),
+        params, modifiers, theme = theme
+      )
+    }
+    if (length(index$line) > 0) {
+      lines <- GeomLine$use_defaults(
+        vec_slice(data, index$line),
+        params, modifiers, theme = theme
+      )
+    }
+    other_default <- modify_list(
+      GeomPolygon$default_aes,
+      aes(
+        fill   = from_theme(fill %||% col_mix(ink, paper, 0.899)),
+        colour = from_theme(colour %||% col_mix(ink, paper, 0.35)),
+        linewidth = from_theme(0.4 * borderwidth)
+      )
+    )
+    if (length(index$other) > 0) {
+      others <- GeomPolygon$use_defaults(
+        vec_slice(data, index$other),
+        params, modifiers,
+        default_aes = other_default,
+        theme = theme
+      )
+    }
+    if (length(index$collection) > 0) {
+      modified <- rename(
+        GeomPoint$default_aes,
+        c(fill = "point_fill")
+      )
+      modified <- modify_list(other_default, modified)
+      collections <- Geom$use_defaults(
+        vec_slice(data, index$collection),
+        params, modifiers,
+        default_aes = modified,
+        theme = theme
+      )
+    }
+
+    # Recombine data in original order
+    data <- vec_c(points, lines, others, collections)
+    vec_slice(data, order(unlist(index)))
+  },
+
   draw_panel = function(self, data, panel_params, coord, legend = NULL,
                         lineend = "butt", linejoin = "round", linemitre = 10,
-                        arrow = NULL, na.rm = TRUE) {
+                        arrow = NULL, arrow.fill = NULL, na.rm = TRUE) {
     if (!inherits(coord, "CoordSf")) {
       cli::cli_abort("{.fn {snake_class(self)}} can only be used with {.fn coord_sf}.")
     }
+    data$shape <- translate_shape_string(data$shape)
 
-    # Need to refactor this to generate one grob per geometry type
-    coord <- coord$transform(data, panel_params)
-    sf_grob(coord, lineend = lineend, linejoin = linejoin, linemitre = linemitre,
-            arrow = arrow, na.rm = na.rm)
+    data <- coord$transform(data, panel_params)
+
+    type <- sf_types[sf::st_geometry_type(data$geometry)]
+    is_point <- type == "point"
+    is_line  <- type == "line"
+    is_collection <- type == "collection"
+
+    fill <- fill_alpha(data$fill %||% rep(NA, nrow(data)), data$alpha)
+    fill[is_line] <- arrow.fill %||% fill[is_line]
+
+    colour <- data$colour
+    colour[is_point | is_line] <-
+      alpha(colour[is_point | is_line], data$alpha[is_point | is_line])
+
+    point_size <- data$size
+    point_size[!(is_point | is_collection)] <-
+      data$linewidth[!(is_point | is_collection)]
+
+    stroke <- (data$stroke %||% rep(0.5, nrow(data))) * .stroke / 2
+    font_size <- point_size * .pt + stroke
+
+    linewidth <- data$linewidth * .pt
+    linewidth[is_point] <- stroke[is_point]
+
+    gp <- gpar(
+      col = colour, fill = fill, fontsize = font_size,
+      lwd = linewidth, lty = data$linetype,
+      lineend = lineend, linejoin = linejoin, linemitre = linemitre
+    )
+
+    sf::st_as_grob(data$geometry, pch = data$shape, gp = gp, arrow = arrow)
   },
 
   draw_key = function(data, params, size) {
-    data <- modify_list(default_aesthetics(params$legend), data)
-    if (params$legend == "point") {
-      draw_key_point(data, params, size)
-    } else if (params$legend == "line") {
-      draw_key_path(data, params, size)
-    } else {
+    switch(
+      params$legend %||% "other",
+      point = draw_key_point(data, params, size),
+      line  = draw_key_path(data, params, size),
       draw_key_polygon(data, params, size)
+    )
+  },
+
+  handle_na = function(self, data, params) {
+    remove <- rep(FALSE, nrow(data))
+
+    types <- sf_types[sf::st_geometry_type(data$geometry)]
+    types <- split(seq_along(remove), types)
+
+    get_missing <- function(geom) {
+      detect_missing(data, c(geom$required_aes, geom$non_missing_aes))
     }
+
+    remove[types$point] <- get_missing(GeomPoint)[types$point]
+    remove[types$line]  <- get_missing(GeomPath)[types$line]
+    remove[types$other] <- get_missing(GeomPolygon)[types$other]
+
+    remove <- remove | get_missing(self)
+
+    if (any(remove)) {
+      data <- vec_slice(data, !remove)
+      if (!isTRUE(params$na.rm)) {
+        cli::cli_warn(
+          "Removed {sum(remove)} row{?s} containing missing values or values \\
+          outside the scale range ({.fn {snake_class(self)}})."
+        )
+      }
+    }
+
+    data
   }
 )
 
-default_aesthetics <- function(type) {
-  if (type == "point") {
-    GeomPoint$default_aes
-  } else if (type == "line") {
-    GeomLine$default_aes
-  } else  {
-    modify_list(GeomPolygon$default_aes, list(fill = "grey90", colour = "grey35"))
-  }
-}
-
-sf_grob <- function(x, lineend = "butt", linejoin = "round", linemitre = 10,
-                    arrow = NULL, na.rm = TRUE) {
-  type <- sf_types[sf::st_geometry_type(x$geometry)]
-  is_point <- type == "point"
-  is_line <- type == "line"
-  is_other <- type == "other"
-  is_collection <- type == "collection"
-  type_ind <- match(type, c("point", "line", "other", "collection"))
-  remove <- rep_len(FALSE, nrow(x))
-  remove[is_point] <- detect_missing(x, c(GeomPoint$required_aes, GeomPoint$non_missing_aes))[is_point]
-  remove[is_line] <- detect_missing(x, c(GeomPath$required_aes, GeomPath$non_missing_aes))[is_line]
-  remove[is_other] <- detect_missing(x, c(GeomPolygon$required_aes, GeomPolygon$non_missing_aes))[is_other]
-  if (any(remove)) {
-    if (!na.rm) {
-      cli::cli_warn(paste0(
-        "Removed {sum(remove)} row{?s} containing missing values or values ",
-        "outside the scale range ({.fn geom_sf})."
-      ))
-    }
-    x <- x[!remove, , drop = FALSE]
-    type_ind <- type_ind[!remove]
-    is_collection <- is_collection[!remove]
-  }
-  defaults <- list(
-    GeomPoint$default_aes,
-    GeomLine$default_aes,
-    modify_list(GeomPolygon$default_aes, list(fill = "grey90", colour = "grey35", linewidth = 0.2))
-  )
-  defaults[[4]] <- modify_list(
-    defaults[[3]],
-    rename(GeomPoint$default_aes, c(size = "point_size", fill = "point_fill"))
-  )
-  default_names <- unique0(unlist(lapply(defaults, names)))
-  defaults <- lapply(setNames(default_names, default_names), function(n) {
-    unlist(lapply(defaults, function(def) def[[n]] %||% NA))
-  })
-  alpha <- x$alpha %||% defaults$alpha[type_ind]
-  col <- x$colour %||% defaults$colour[type_ind]
-  col[is_point | is_line] <- alpha(col[is_point | is_line], alpha[is_point | is_line])
-  fill <- x$fill %||% defaults$fill[type_ind]
-  fill <- fill_alpha(fill, alpha)
-  size <- x$size %||% defaults$size[type_ind]
-  linewidth <- x$linewidth %||% defaults$linewidth[type_ind]
-  point_size <- ifelse(
-    is_collection,
-    x$size %||% defaults$point_size[type_ind],
-    ifelse(is_point, size, linewidth)
-  )
-  stroke <- (x$stroke %||% defaults$stroke[1]) * .stroke / 2
-  fontsize <- point_size * .pt + stroke
-  lwd <- ifelse(is_point, stroke, linewidth * .pt)
-  pch <- x$shape %||% defaults$shape[type_ind]
-  lty <- x$linetype %||% defaults$linetype[type_ind]
-  gp <- gpar(
-    col = col, fill = fill, fontsize = fontsize, lwd = lwd, lty = lty,
-    lineend = lineend, linejoin = linejoin, linemitre = linemitre
-  )
-  sf::st_as_grob(x$geometry, pch = pch, gp = gp, arrow = arrow)
-}
-
 #' @export
 #' @rdname ggsf
 #' @inheritParams geom_point
@@ -255,28 +312,25 @@ geom_sf <- function(mapping = aes(), data = NULL, stat = "sf",
 #' @inheritParams geom_label
 #' @inheritParams stat_sf_coordinates
 geom_sf_label <- function(mapping = aes(), data = NULL,
-                          stat = "sf_coordinates", position = "identity",
+                          stat = "sf_coordinates", position = "nudge",
                           ...,
                           parse = FALSE,
-                          nudge_x = 0,
-                          nudge_y = 0,
                           label.padding = unit(0.25, "lines"),
                           label.r = unit(0.15, "lines"),
-                          label.size = 0.25,
+                          label.size = deprecated(),
+                          border.colour = NULL,
+                          border.color = NULL,
+                          text.colour = NULL,
+                          text.color = NULL,
                           na.rm = FALSE,
                           show.legend = NA,
                           inherit.aes = TRUE,
                           fun.geometry = NULL) {
 
-  if (!missing(nudge_x) || !missing(nudge_y)) {
-    if (!missing(position)) {
-      cli::cli_abort(c(
-        "Both {.arg position} and {.arg nudge_x}/{.arg nudge_y} are supplied.",
-        "i" = "Only use one approach to alter the position."
-      ))
-    }
-
-    position <- position_nudge(nudge_x, nudge_y)
+  extra_args <- list2(...)
+  if (lifecycle::is_present(label.size)) {
+    deprecate_warn0("3.5.0", "geom_label(label.size)", "geom_label(linewidth)")
+    extra_args$linewidth <- extra_args$linewidth %||% label.size
   }
 
   layer_sf(
@@ -291,10 +345,11 @@ geom_sf_label <- function(mapping = aes(), data = NULL,
       parse = parse,
       label.padding = label.padding,
       label.r = label.r,
-      label.size = label.size,
       na.rm = na.rm,
       fun.geometry = fun.geometry,
-      ...
+      border.colour = border.color %||% border.colour,
+      text.colour = text.color %||% text.colour,
+      !!!extra_args
     )
   )
 }
@@ -304,28 +359,15 @@ geom_sf_label <- function(mapping = aes(), data = NULL,
 #' @inheritParams geom_text
 #' @inheritParams stat_sf_coordinates
 geom_sf_text <- function(mapping = aes(), data = NULL,
-                         stat = "sf_coordinates", position = "identity",
+                         stat = "sf_coordinates", position = "nudge",
                          ...,
                          parse = FALSE,
-                         nudge_x = 0,
-                         nudge_y = 0,
                          check_overlap = FALSE,
                          na.rm = FALSE,
                          show.legend = NA,
                          inherit.aes = TRUE,
                          fun.geometry = NULL) {
 
-  if (!missing(nudge_x) || !missing(nudge_y)) {
-    if (!missing(position)) {
-      cli::cli_abort(c(
-        "Both {.arg position} and {.arg nudge_x}/{.arg nudge_y} are supplied.",
-        "i" = "Only use one approach to alter the position."
-      ))
-    }
-
-    position <- position_nudge(nudge_x, nudge_y)
-  }
-
   layer_sf(
     data = data,
     mapping = mapping,
