@@ -6,7 +6,7 @@
 #' no inherent order, this use is not advised.
 #'
 #' @inheritParams discrete_scale
-#' @inheritDotParams discrete_scale -expand -position -na.value -scale_name -palette
+#' @inheritDotParams discrete_scale -expand -position -na.value -scale_name -palette -fallback.palette
 #' @rdname scale_linetype
 #' @details
 #' Lines can be referred to by number, name or hex code. Contrary to base R
@@ -45,7 +45,7 @@
 scale_linetype <- function(name = waiver(), ..., aesthetics = "linetype") {
   discrete_scale(
     aesthetics, name = name,
-    palette = NULL,
+    palette = NULL, fallback.palette = pal_linetype(),
     ...
   )
 }
@@ -55,7 +55,7 @@ scale_linetype <- function(name = waiver(), ..., aesthetics = "linetype") {
 scale_linetype_binned <- function(name = waiver(), ..., aesthetics = "linetype") {
   binned_scale(
     aesthetics, name = name,
-    palette = NULL,
+    palette = NULL, fallback.palette = pal_binned(pal_linetype()),
     ...
   )
 }
