@@ -1,3 +1,28 @@
+#' @rdname Stat
+#' @format NULL
+#' @usage NULL
+#' @export
+StatEllipse <- ggproto(
+  "StatEllipse", Stat,
+  required_aes = c("x", "y"),
+  optional_aes = "weight",
+  dropped_aes = "weight",
+
+  setup_params = function(data, params) {
+    params$type <- params$type %||% "t"
+    if (identical(params$type, "t")) {
+      check_installed("MASS", "for calculating ellipses with `type = \"t\"`.")
+    }
+    params
+  },
+
+  compute_group = function(data, scales, type = "t", level = 0.95,
+                           segments = 51, na.rm = FALSE) {
+    calculate_ellipse(data = data, vars = c("x", "y"), type = type,
+                      level = level, segments = segments)
+  }
+)
+
 #' Compute normal data ellipses
 #'
 #' The method for calculating the ellipses has been modified from
@@ -20,6 +45,7 @@
 #' @param segments The number of segments to be used in drawing the ellipse.
 #' @inheritParams layer
 #' @inheritParams geom_point
+#' @aesthetics StatEllipse
 #' @export
 #' @examples
 #' ggplot(faithful, aes(waiting, eruptions)) +
@@ -43,51 +69,15 @@
 #'
 #' ggplot(faithful, aes(waiting, eruptions, fill = eruptions > 3)) +
 #'   stat_ellipse(geom = "polygon")
-stat_ellipse <- function(mapping = NULL, data = NULL,
-                         geom = "path", position = "identity",
-                         ...,
-                         type = "t",
-                         level = 0.95,
-                         segments = 51,
-                         na.rm = FALSE,
-                         show.legend = NA,
-                         inherit.aes = TRUE) {
-  layer(
-    data = data,
-    mapping = mapping,
-    stat = StatEllipse,
-    geom = geom,
-    position = position,
-    show.legend = show.legend,
-    inherit.aes = inherit.aes,
-    params = list2(
-      type = type,
-      level = level,
-      segments = segments,
-      na.rm = na.rm,
-      ...
-    )
-  )
-}
-
-#' @rdname ggplot2-ggproto
-#' @format NULL
-#' @usage NULL
-#' @export
-StatEllipse <- ggproto("StatEllipse", Stat,
-  required_aes = c("x", "y"),
-
-  compute_group = function(data, scales, type = "t", level = 0.95,
-                           segments = 51, na.rm = FALSE) {
-    calculate_ellipse(data = data, vars = c("x", "y"), type = type,
-                      level = level, segments = segments)
-  }
-)
+stat_ellipse <- make_constructor(StatEllipse, geom = "path")
 
 calculate_ellipse <- function(data, vars, type, level, segments){
   dfn <- 2
   dfd <- nrow(data) - 1
 
+  weight <- data$weight %||% rep(1, nrow(data))
+  weight <- weight / sum(weight)
+
   if (!type %in% c("t", "norm", "euclid")) {
     cli::cli_inform("Unrecognized ellipse type")
     ellipse <- matrix(NA_real_, ncol = 2)
@@ -96,11 +86,12 @@ calculate_ellipse <- function(data, vars, type, level, segments){
     ellipse <- matrix(NA_real_, ncol = 2)
   } else {
     if (type == "t") {
-      v <- MASS::cov.trob(data[,vars])
+      # Prone to convergence problems when `sum(weight) != nrow(data)`
+      v <- MASS::cov.trob(data[,vars], wt = weight * nrow(data))
     } else if (type == "norm") {
-      v <- stats::cov.wt(data[,vars])
+      v <- stats::cov.wt(data[,vars], wt = weight)
     } else if (type == "euclid") {
-      v <- stats::cov.wt(data[,vars])
+      v <- stats::cov.wt(data[,vars], wt = weight)
       v$cov <- diag(rep(min(diag(v$cov)), 2))
     }
     shape <- v$cov
