@@ -33,7 +33,7 @@
 #' @section Coord summary:
 #'
 #' The function `summarise_coord()` returns information about the log base for
-#' coordinates that are log-transformed in `coord_trans()`, and it also indicates
+#' coordinates that are log-transformed in `coord_transform()`, and it also indicates
 #' whether the coord has flipped the x and y axes.
 #'
 #' @section Layer summary:
@@ -44,7 +44,10 @@
 #' @param p A ggplot_built object.
 #'
 #' @examples
-#' p <- ggplot(mpg, aes(displ, hwy)) + geom_point() + facet_wrap(~class)
+#' p <-
+#'   ggplot(mpg, aes(displ, hwy)) +
+#'   geom_point() +
+#'   facet_wrap(~class)
 #' b <- ggplot_build(p)
 #'
 #' summarise_layout(b)
@@ -58,12 +61,12 @@ NULL
 
 #' @rdname summarise_plot
 #' @export
-summarise_layout = function(p) {
-  if (!inherits(p, "ggplot_built")) abort("`p` must be a ggplot_build object")
-  l <- p$layout
+summarise_layout <- function(p) {
+  check_inherits(p, "ggplot2::ggplot_built")
+  l <- p@layout
 
   layout <- l$layout
-  layout <- tibble(
+  layout <- data_frame0(
     panel = l$layout$PANEL,
     row   = l$layout$ROW,
     col   = l$layout$COL
@@ -95,8 +98,8 @@ summarise_layout = function(p) {
 
 #' @rdname summarise_plot
 #' @export
-summarise_coord = function(p) {
-  if (!inherits(p, "ggplot_built")) abort("`p` must be a ggplot_build object")
+summarise_coord <- function(p) {
+  check_inherits(p, "ggplot2::ggplot_built")
 
   # Given a transform object, find the log base; if the transform object is
   # NULL, or if it's not a log transform, return NA.
@@ -109,9 +112,9 @@ summarise_coord = function(p) {
   }
 
   list(
-    xlog = trans_get_log_base(p$layout$coord$trans$x),
-    ylog = trans_get_log_base(p$layout$coord$trans$y),
-    flip = inherits(p$layout$coord, "CoordFlip")
+    xlog = trans_get_log_base(p@layout$coord$trans$x),
+    ylog = trans_get_log_base(p@layout$coord$trans$y),
+    flip = inherits(p@layout$coord, "CoordFlip")
   )
 }
 
@@ -119,19 +122,19 @@ summarise_coord = function(p) {
 #' @rdname summarise_plot
 #' @export
 summarise_layers <- function(p) {
-  if (!inherits(p, "ggplot_built")) abort("`p` must be a ggplot_build object")
+  check_inherits(p, "ggplot2::ggplot_built")
 
-  # Default mappings. Make sure it's a regular list instead of an uneval
+  # Default mappings. Make sure it's a regular list instead of a mapping
   # object.
-  default_mapping <- unclass(p$plot$mapping)
+  default_mapping <- unclass(p@plot@mapping)
 
-  layer_mappings <- lapply(p$plot$layers, function(layer) {
+  layer_mappings <- lapply(p@plot@layers, function(layer) {
     defaults(layer$mapping, default_mapping)
   })
 
   # This currently only returns the mappings, but in the future, other
   # information could be added.
-  tibble(
+  data_frame0(
     mapping = layer_mappings
   )
 }
