@@ -5,7 +5,7 @@ test_that("NA's don't appear in breaks", {
     ns <- names(trained)[grepl("(\\.major)|(\\.minor)$", names(trained))]
 
     for (n in ns) {
-      if (!is.null(trained[n]) && any(is.na(trained[n])))
+      if (!is.null(trained[n]) && anyNA(trained[n]))
         return(TRUE)
     }
 
@@ -19,13 +19,13 @@ test_that("NA's don't appear in breaks", {
   # This is a test to make sure the later tests will be useful!
   # It's possible that changes to the way that breaks are calculated will
   # make it so that scale_break_positions will no longer give NA for range 1, 12
-  expect_true(any(is.na(scale_x$break_positions())))
-  expect_true(any(is.na(scale_y$break_positions())))
+  expect_true(anyNA(scale_x$break_positions()))
+  expect_true(anyNA(scale_y$break_positions()))
 
   # Check the various types of coords to make sure they don't have NA breaks
   expect_false(any_NA_major_minor(coord_polar()$setup_panel_params(scale_x, scale_y)))
   expect_false(any_NA_major_minor(coord_cartesian()$setup_panel_params(scale_x, scale_y)))
-  expect_false(any_NA_major_minor(coord_trans()$setup_panel_params(scale_x, scale_y)))
+  expect_false(any_NA_major_minor(coord_transform()$setup_panel_params(scale_x, scale_y)))
   expect_false(any_NA_major_minor(coord_fixed()$setup_panel_params(scale_x, scale_y)))
 
   skip_if_not_installed("mapproj")
