@@ -1,5 +1,3 @@
-context("geom-sf")
-
 test_that("geom_sf() determines the legend type automatically", {
   skip_if_not_installed("sf")
   if (packageVersion("sf") < "0.5.3") skip("Need sf 0.5.3")
@@ -32,24 +30,24 @@ test_that("geom_sf() determines the legend type automatically", {
   }
 
   # test the automatic choice
-  expect_identical(fun_geom_sf(mp, TRUE)$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mp, TRUE)$plot$layers[[1]]$computed_geom_params$legend, "point")
+  expect_true(fun_geom_sf(mp, TRUE)@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mp, TRUE)@plot@layers[[1]]$computed_geom_params$legend, "point")
 
-  expect_identical(fun_geom_sf(mls, TRUE)$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mls, TRUE)$plot$layers[[1]]$computed_geom_params$legend, "line")
+  expect_true(fun_geom_sf(mls, TRUE)@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mls, TRUE)@plot@layers[[1]]$computed_geom_params$legend, "line")
 
-  expect_identical(fun_geom_sf(mpol, TRUE)$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mpol, TRUE)$plot$layers[[1]]$computed_geom_params$legend, "polygon")
+  expect_true(fun_geom_sf(mpol, TRUE)@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mpol, TRUE)@plot@layers[[1]]$computed_geom_params$legend, "other")
 
   # test that automatic choice can be overridden manually
-  expect_identical(fun_geom_sf(mp, "point")$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mp, "point")$plot$layers[[1]]$computed_geom_params$legend, "point")
+  expect_true(fun_geom_sf(mp, "point")@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mp, "point")@plot@layers[[1]]$computed_geom_params$legend, "point")
 
-  expect_identical(fun_geom_sf(mls, "point")$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mls, "point")$plot$layers[[1]]$computed_geom_params$legend, "point")
+  expect_true(fun_geom_sf(mls, "point")@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mls, "point")@plot@layers[[1]]$computed_geom_params$legend, "point")
 
-  expect_identical(fun_geom_sf(mpol, "point")$plot$layers[[1]]$show.legend, TRUE)
-  expect_identical(fun_geom_sf(mpol, "point")$plot$layers[[1]]$computed_geom_params$legend, "point")
+  expect_true(fun_geom_sf(mpol, "point")@plot@layers[[1]]$show.legend)
+  expect_identical(fun_geom_sf(mpol, "point")@plot@layers[[1]]$computed_geom_params$legend, "point")
 })
 
 test_that("geom_sf() determines the legend type from mapped geometry column", {
@@ -70,19 +68,12 @@ test_that("geom_sf() determines the legend type from mapped geometry column", {
   p <- ggplot_build(
     ggplot(d_sf) + geom_sf(aes(geometry = g_point, colour = "a"))
   )
-  expect_identical(p$plot$layers[[1]]$computed_geom_params$legend, "point")
+  expect_identical(p@plot@layers[[1]]$computed_geom_params$legend, "point")
 
   p <- ggplot_build(
     ggplot(d_sf) + geom_sf(aes(geometry = g_line, colour = "a"))
   )
-  expect_identical(p$plot$layers[[1]]$computed_geom_params$legend, "line")
-
-  # If `geometry` is not a symbol, `LayerSf$setup_layer()` gives up guessing
-  # the legend type, and falls back to "polygon"
-  p <- ggplot_build(
-    ggplot(d_sf) + geom_sf(aes(geometry = identity(g_point), colour = "a"))
-  )
-  expect_identical(p$plot$layers[[1]]$computed_geom_params$legend, "polygon")
+  expect_identical(p@plot@layers[[1]]$computed_geom_params$legend, "line")
 })
 
 test_that("geom_sf() removes rows containing missing aes", {
@@ -90,7 +81,7 @@ test_that("geom_sf() removes rows containing missing aes", {
   if (packageVersion("sf") < "0.5.3") skip("Need sf 0.5.3")
 
   grob_xy_length <- function(x) {
-    g <- layer_grob(x)[[1]]
+    g <- get_layer_grob(x)[[1]]
     c(length(g$x), length(g$y))
   }
 
@@ -101,20 +92,17 @@ test_that("geom_sf() removes rows containing missing aes", {
     colour = c("red", NA)
   )
 
-  p <- ggplot(pts) + geom_sf()
-  expect_warning(
-    expect_identical(grob_xy_length(p + aes(size = size)), c(1L, 1L)),
-    "Removed 1 rows containing missing values"
+  p <- ggplot(pts)
+  expect_snapshot_warning(
+    expect_identical(grob_xy_length(p + geom_sf(aes(size = size))), c(1L, 1L))
   )
-  expect_warning(
-    expect_identical(grob_xy_length(p + aes(shape = shape)), c(1L, 1L)),
-    "Removed 1 rows containing missing values"
+  expect_snapshot_warning(
+    expect_identical(grob_xy_length(p + geom_sf(aes(shape = shape))), c(1L, 1L))
   )
   # default colour scale maps a colour even to a NA, so identity scale is needed to see if NA is removed
-  expect_warning(
-    expect_identical(grob_xy_length(p + aes(colour = colour) + scale_colour_identity()),
-                     c(1L, 1L)),
-    "Removed 1 rows containing missing values"
+  expect_snapshot_warning(
+    expect_identical(grob_xy_length(p + geom_sf(aes(colour = colour)) + scale_colour_identity()),
+                     c(1L, 1L))
   )
 })
 
@@ -129,7 +117,7 @@ test_that("geom_sf() handles alpha properly", {
   )
   red <- "#FF0000FF"
   p <- ggplot(sfc) + geom_sf(colour = red, fill = red, alpha = 0.5)
-  g <- layer_grob(p)[[1]]
+  g <- get_layer_grob(p)[[1]]
 
   # alpha affects the colour of points and lines
   expect_equal(g[[1]]$gp$col, alpha(red, 0.5))
@@ -139,6 +127,30 @@ test_that("geom_sf() handles alpha properly", {
   expect_equal(g[[3]]$gp$fill, alpha(red, 0.5))
 })
 
+test_that("errors are correctly triggered", {
+  skip_if_not_installed("sf")
+  pts <- sf::st_sf(
+    geometry = sf::st_sfc(sf::st_point(0:1), sf::st_point(1:2)),
+    size = c(1, NA),
+    shape = c("a", NA),
+    colour = c("red", NA)
+  )
+  p <- ggplot(pts) + geom_sf() + coord_cartesian()
+  expect_snapshot_error(ggplotGrob(p))
+  expect_snapshot_warning(geom_sf_label(position = "jitter", nudge_x = 0.5))
+  expect_snapshot_warning(geom_sf_text(position = "jitter", nudge_x = 0.5))
+
+  # #5204: missing linewidth should be dropped
+  pts <- sf::st_sf(
+    geometry = sf::st_sfc(
+      sf::st_linestring(matrix(c(0, 1, 0, 1), ncol = 2)),
+      sf::st_linestring(matrix(c(0, 1, 1, 0), ncol = 2))
+    ),
+    linewidth = c(1, NA)
+  )
+  expect_snapshot_warning(GeomSf$handle_na(pts, list(na.rm = FALSE)))
+})
+
 # Visual tests ------------------------------------------------------------
 
 test_that("geom_sf draws correctly", {
@@ -162,10 +174,10 @@ test_that("geom_sf draws correctly", {
   # Perform minimal tests
   pts <- sf::st_sf(a = 1:2, geometry = sf::st_sfc(sf::st_point(0:1), sf::st_point(1:2)))
   plot <- ggplot() + geom_sf(data = pts)
-  expect_error(regexp = NA, ggplot_build(plot))
+  expect_no_error(ggplot_build(plot))
 
   expect_doppelganger("North Carolina county boundaries",
-    ggplot() + geom_sf(data = nc) + coord_sf(datum = 4326)
+    ggplot() + geom_sf(data = nc, linetype = 2) + coord_sf(datum = 4326)
   )
 
   pts <- sf::st_sf(a = 1:2, geometry = sf::st_sfc(sf::st_point(0:1), sf::st_point(1:2)))
@@ -174,6 +186,84 @@ test_that("geom_sf draws correctly", {
   )
 })
 
+test_that("geom_sf data type renders appropriate legends", {
+  skip_if_not_installed("sf")
+  p <- ggplot() + geom_sf(aes(colour = col))
+
+  # Point data
+  data <- sf::st_as_sf(
+    data.frame(lon = c(1, 2), lat = c(3, 4), col = c("foo", "bar")),
+    coords = c("lon", "lat")
+  )
+  expect_doppelganger(
+    "geom_sf point legend",
+    ggplot_add(data, p)
+  )
+
+  # Line data
+  data <- sf::st_as_sf(
+    sf::st_sfc(
+      sf::st_linestring(x = cbind(1:2, 3:4)),
+      sf::st_linestring(x = cbind(3:4, 5:6))
+    ),
+    col = c("foo", "bar")
+  )
+  expect_doppelganger(
+    "geom_sf line legend",
+    ggplot_add(data, p)
+  )
+
+  # Polygon data
+  data <- sf::st_as_sf(
+    sf::st_sfc(
+      sf::st_polygon(list(cbind(c(1, 2, 2, 1), c(3, 3, 4, 3)))),
+      sf::st_polygon(list(cbind(c(3, 3, 4, 3), c(5, 6, 6, 5))))
+    ),
+    col = c("foo", "bar")
+  )
+  expect_doppelganger(
+    "geom_sf polygon legend",
+    ggplot_add(data, p)
+  )
+})
+
+test_that("geom_sf uses combinations of geometry correctly", {
+  skip_if_not_installed("sf")
+
+  t <- seq(0, 2 *pi, length.out = 10)
+  data <- sf::st_sf(sf::st_sfc(
+    sf::st_multipoint(cbind(1:2, 3:4)),
+    sf::st_multilinestring(list(
+      cbind(c(1, 1.8), c(3.8, 3)),
+      cbind(c(1.2, 2), c(4, 3.2))
+    )),
+    sf::st_polygon(list(
+      cbind(cos(t), zapsmall(sin(t))),
+      cbind(cos(t), zapsmall(sin(t))) + 5
+    )),
+    sf::st_geometrycollection(x = list(
+      sf::st_point(x = c(3, 2)),
+      sf::st_linestring(cbind(c(2, 4, 4), c(1, 1, 3)))
+    )),
+    sf::st_linestring(x = cbind(c(2, 6), c(-1, 3))),
+    sf::st_point(c(5, 0))
+  ))
+
+  update_geom_defaults("point", list(colour = "blue"))
+  update_geom_defaults("line", list(colour = "red"))
+  # Note: polygon defaults are mostly ignored or overridden
+
+  withr::defer({
+    update_geom_defaults("point", NULL)
+    update_geom_defaults("line",  NULL)
+  })
+
+  expect_doppelganger(
+    "mixed geometry types",
+    ggplot(data) + geom_sf()
+  )
+})
+
 test_that("geom_sf_text() and geom_sf_label() draws correctly", {
   skip_if_not_installed("sf")
   if (packageVersion("sf") < "0.5.3") skip("Need sf 0.5.3")
@@ -202,3 +292,38 @@ test_that("geom_sf_text() and geom_sf_label() draws correctly", {
     ggplot() + geom_sf_label(data = nc_3857, aes(label = NAME))
   )
 })
+
+test_that("geom_sf draws arrows correctly", {
+  skip_if_not_installed("sf")
+  if (packageVersion("sf") < "0.5.3") skip("Need sf 0.5.3")
+
+  nc_tiny_coords <- data_frame(
+    x = c(-81.473, -81.741, -81.67, -81.345, -81.266, -81.24, -81.473),
+    y = c(36.234, 36.392, 36.59, 36.573, 36.437, 36.365, 36.234)
+  )
+
+  nc <- sf::st_linestring(
+      sf::st_coordinates(sf::st_as_sf(nc_tiny_coords, coords = c("x", "y"), crs = 4326))
+    )
+
+  nc2 <- sf::st_cast(
+    sf::st_sfc(
+      sf::st_multilinestring(lapply(
+        1:(length(sf::st_coordinates(nc)[, 1]) - 1),
+          function(x) rbind(
+            as.numeric(sf::st_coordinates(nc)[x, 1:2]),
+            as.numeric(sf::st_coordinates(nc)[x + 1, 1:2])
+            )
+        )
+      )
+    ), "LINESTRING"
+  )
+
+  expect_doppelganger("North Carolina county boundaries with arrow",
+    ggplot() + geom_sf(data = nc, arrow = arrow()) + coord_sf(datum = 4326)
+  )
+
+  expect_doppelganger("North Carolina county boundaries with more than one arrow",
+    ggplot() + geom_sf(data = nc2, arrow = arrow()) + coord_sf(datum = 4326)
+  )
+})
